package com.tzld.piaoquan.recommend.server.service.rank.strategy;

import com.alibaba.fastjson.JSON;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.recommend.server.common.ThreadPoolFactory;
import com.tzld.piaoquan.recommend.server.common.base.RankItem;
import com.tzld.piaoquan.recommend.server.model.MachineInfo;
import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.FeatureService;
import com.tzld.piaoquan.recommend.server.service.rank.RankParam;
import com.tzld.piaoquan.recommend.server.service.rank.bo.UserSRBO;
import com.tzld.piaoquan.recommend.server.service.rank.bo.UserShareReturnProfile;
import com.tzld.piaoquan.recommend.server.service.rank.tansform.FeatureV6;
import com.tzld.piaoquan.recommend.server.service.recall.strategy.*;
import com.tzld.piaoquan.recommend.server.service.score.ScorerUtils;
import com.tzld.piaoquan.recommend.server.util.CommonCollectionUtils;
import com.tzld.piaoquan.recommend.server.util.FeatureBucketUtils;
import com.tzld.piaoquan.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service
@Slf4j
public class RankStrategy4RegionMergeModelV564 extends RankStrategy4RegionMergeModelBasic {
    @ApolloJsonValue("${rank.score.merge.weightv564:}")
    private Map<String, Double> mergeWeight;

    @Autowired
    private FeatureService featureService;

    @Override
    public List<Video> mergeAndRankRovRecall(RankParam param) {
        Map<String, Double> mergeWeight = this.mergeWeight != null ? this.mergeWeight : new HashMap<>(0);
        //-------------------融-------------------
        //-------------------合-------------------
        //-------------------逻-------------------
        //-------------------辑-------------------

        long currentMs = System.currentTimeMillis();
        List<Video> oldRovs = new ArrayList<>();
        oldRovs.addAll(extractAndSort(param, RegionHRecallStrategy.PUSH_FORM));
        oldRovs.addAll(extractAndSort(param, RegionHDupRecallStrategy.PUSH_FORM));
        oldRovs.addAll(extractAndSort(param, Region24HRecallStrategy.PUSH_FORM));
        oldRovs.addAll(extractAndSort(param, RegionRelative24HRecallStrategy.PUSH_FORM));
        oldRovs.addAll(extractAndSort(param, RegionRelative24HDupRecallStrategy.PUSH_FORM));
        removeDuplicate(oldRovs);
        int sizeReturn = param.getSize();
        List<Video> v0 = oldRovs.size() <= sizeReturn
                ? oldRovs
                : oldRovs.subList(0, sizeReturn);
        Set<Long> setVideo = new HashSet<>();
        this.duplicate(setVideo, v0);
        setVideo.addAll(v0.stream().map(Video::getVideoId).collect(Collectors.toSet()));
        List<Video> rovRecallRank = new ArrayList<>(v0);
        //-------------------return相似召回------------------
        List<Video> v6 = extractAndSort(param, ReturnVideoRecallStrategy.PUSH_FORM);
        v6 = v6.stream().filter(r -> !setVideo.contains(r.getVideoId())).collect(Collectors.toList());
        v6 = v6.subList(0, Math.min(mergeWeight.getOrDefault("v6", 5.0).intValue(), v6.size()));
        rovRecallRank.addAll(v6);
        setVideo.addAll(v6.stream().map(Video::getVideoId).collect(Collectors.toSet()));
        //-------------------新地域召回------------------
        List<Video> v1 = extractAndSort(param, RegionRealtimeRecallStrategyV1.PUSH_FORM);
        v1 = v1.stream().filter(r -> !setVideo.contains(r.getVideoId())).collect(Collectors.toList());
        v1 = v1.subList(0, Math.min(mergeWeight.getOrDefault("v1", 5.0).intValue(), v1.size()));
        rovRecallRank.addAll(v1);
        setVideo.addAll(v1.stream().map(Video::getVideoId).collect(Collectors.toSet()));
        //-------------------scene cf rovn------------------
        List<Video> sceneCFRovn = extractAndSort(param, SceneCFRovnRecallStrategy.PUSH_FORM);
        sceneCFRovn = sceneCFRovn.stream().filter(r -> !setVideo.contains(r.getVideoId())).collect(Collectors.toList());
        sceneCFRovn = sceneCFRovn.subList(0, Math.min(mergeWeight.getOrDefault("sceneCFRovn", 5.0).intValue(), sceneCFRovn.size()));
        rovRecallRank.addAll(sceneCFRovn);
        setVideo.addAll(sceneCFRovn.stream().map(Video::getVideoId).collect(Collectors.toSet()));
        //-------------------scene cf rosn------------------
        List<Video> sceneCFRosn = extractAndSort(param, SceneCFRosnRecallStrategy.PUSH_FORM);
        sceneCFRosn = sceneCFRosn.stream().filter(r -> !setVideo.contains(r.getVideoId())).collect(Collectors.toList());
        sceneCFRosn = sceneCFRosn.subList(0, Math.min(mergeWeight.getOrDefault("sceneCFRosn", 5.0).intValue(), sceneCFRosn.size()));
        rovRecallRank.addAll(sceneCFRosn);
        setVideo.addAll(sceneCFRosn.stream().map(Video::getVideoId).collect(Collectors.toSet()));
        // -------------------cate1------------------
        int cate1RecallN = mergeWeight.getOrDefault("cate1RecallN", 5.0).intValue();
        addRecall(param, cate1RecallN, UserCate1RecallStrategy.PUSH_FORM, setVideo, rovRecallRank);
        // -------------------cate2------------------
        int cate2RecallN = mergeWeight.getOrDefault("cate2RecallN", 5.0).intValue();
        addRecall(param, cate2RecallN, UserCate2RecallStrategy.PUSH_FORM, setVideo, rovRecallRank);
        // -------------------head province cate1------------------
        int headCate1RecallN = mergeWeight.getOrDefault("headCate1RecallN", 5.0).intValue();
        addRecall(param, headCate1RecallN, HeadProvinceCate1RecallStrategy.PUSH_FORM, setVideo, rovRecallRank);
        // -------------------head province cate2------------------
        int headCate2RecallN = mergeWeight.getOrDefault("headCate2RecallN", 5.0).intValue();
        addRecall(param, headCate2RecallN, HeadProvinceCate2RecallStrategy.PUSH_FORM, setVideo, rovRecallRank);

        //-------------------排-------------------
        //-------------------序-------------------
        //-------------------逻-------------------
        //-------------------辑-------------------

        // 1. 批量获取特征  省份参数要对齐  headvid  要传递过来！
        // k1:视频、k2:表、k3:特征、v:特征值
        List<String> vids = CommonCollectionUtils.toListDistinct(rovRecallRank, v -> String.valueOf(v.getVideoId()));
        String headVid = String.valueOf(param.getHeadVid());
        Map<String, Map<String, Map<String, String>>> videoBaseInfoMap = featureService.getVideoBaseInfo(headVid, vids);
        FeatureService.Feature feature = featureService.getFeatureV3(param, videoBaseInfoMap, vids);
        Map<String, Map<String, String>> featureOriginUser = feature.getUserFeature();
        Map<String, Map<String, Map<String, String>>> featureOriginVideo = feature.getVideoFeature();
        Map<String, String> headVideoInfo = videoBaseInfoMap.getOrDefault(headVid, new HashMap<>()).getOrDefault("alg_vid_feature_basic_info", new HashMap<>());

        // 2. 用户信息预处理
        Map<String, Map<String, String[]>> newC7Map = FeatureV6.parseUCFScore(featureOriginUser.getOrDefault("alg_mid_feature_sharecf", new HashMap<>()));
        Map<String, Map<String, String[]>> newC8Map = FeatureV6.parseUCFScore(featureOriginUser.getOrDefault("alg_mid_feature_returncf", new HashMap<>()));
        UserShareReturnProfile userProfile = parseUserProfile(featureOriginUser);
        Map<String, Map<String, String>> userBehaviorVideoMap = getUserBehaviorVideoMap(userProfile);

        // 3. 特征处理
        List<RankItem> rankItems = CommonCollectionUtils.toList(rovRecallRank, RankItem::new);
        Map<String, String> userFeatureMap = getUserFeature(currentMs, param, headVideoInfo, userProfile, featureOriginUser);
        batchGetVideoFeature(currentMs, userProfile, headVideoInfo, videoBaseInfoMap,
                newC7Map, newC8Map, featureOriginUser, userBehaviorVideoMap, featureOriginVideo, rankItems);

        // 4. 排序模型计算
        Map<String, String> sceneFeatureMap = new HashMap<>(0);
        List<RankItem> items = ScorerUtils.getScorerPipeline("feeds_score_config_fm_xgb_20250317.conf").scoring(sceneFeatureMap, userFeatureMap, userFeatureMap, rankItems);

        // 5. 排序公式特征
        double xgbRovNegRate = mergeWeight.getOrDefault("xgbRovNegRate", 0.059);
        double xgbNorPowerWeight = mergeWeight.getOrDefault("xgbNorPowerWeight", 1.22);
        double xgbNorPowerExp = mergeWeight.getOrDefault("xgbNorPowerExp", 1.24);
        Map<String, Map<String, String>> vid2MapFeature = this.getVideoRedisFeature(vids, "redis:vid_hasreturn_vor:");
        List<Video> result = new ArrayList<>();
        for (RankItem item : items) {
            double score;
            double fmRovOrigin = item.getScoreRov();
            item.getScoresMap().put("fmRovOrigin", fmRovOrigin);
            double fmRov = restoreScore(fmRovOrigin, xgbRovNegRate);
            item.getScoresMap().put("fmRov", fmRov);
            double hasReturnRovScore = Double.parseDouble(vid2MapFeature.getOrDefault(item.getVideoId() + "", new HashMap<>()).getOrDefault("rov", "0"));
            item.getScoresMap().put("hasReturnRovScore", hasReturnRovScore);
            double norXGBScore = item.getScoresMap().getOrDefault("NorXGBScore", 0d);
            double newNorXGBScore = norPowerCalibration(xgbNorPowerWeight, xgbNorPowerExp, norXGBScore);
            double vor = Double.parseDouble(vid2MapFeature.getOrDefault(item.getVideoId() + "", new HashMap<>()).getOrDefault("vor", "0"));
            item.getScoresMap().put("vor", vor);
            score = fmRov * (0.1 + newNorXGBScore) * (0.1 + vor);
            Video video = item.getVideo();
            video.setScore(score);
            video.setSortScore(score);
            video.setScoresMap(item.getScoresMap());
            video.setAllFeatureMap(item.getAllFeatureMap());
            if (MapUtils.isNotEmpty(feature.getVideoFeature()) && MapUtils.isNotEmpty(feature.getVideoFeature().get(item.getVideoId() + ""))) {
                video.getMetaFeatureMap().putAll(feature.getVideoFeature().get(item.getVideoId() + ""));
            }
            if (MapUtils.isNotEmpty(videoBaseInfoMap) && MapUtils.isNotEmpty(videoBaseInfoMap.get(item.getVideoId() + ""))) {
                video.getMetaFeatureMap().putAll(videoBaseInfoMap.get(item.getVideoId() + ""));
            }
            if (MapUtils.isNotEmpty(headVideoInfo)) {
                video.getMetaFeatureMap().put("head_video", headVideoInfo);
            }
            if (MapUtils.isNotEmpty(feature.getUserFeature())) {
                video.getMetaFeatureMap().putAll(feature.getUserFeature());
            }
            result.add(video);
        }
        result.sort(Comparator.comparingDouble(o -> -o.getSortScore()));
        return result;
    }

    private UserShareReturnProfile parseUserProfile(Map<String, Map<String, String>> userOriginInfo) {
        if (null != userOriginInfo) {
            Map<String, String> c9 = userOriginInfo.get("alg_recsys_feature_user_share_return_stat");
            if (null != c9 && !c9.isEmpty()) {
                String c9Str = JSONUtils.toJson(c9);
                if (!c9Str.isEmpty()) {
                    try {
                        return JSON.parseObject(c9Str, UserShareReturnProfile.class);
                    } catch (Exception e) {
                        log.error("parseObject user profile error! value=[{}]", c9Str, e);
                    }
                }
            }
        }
        return null;
    }

    private Map<String, Map<String, String>> getUserBehaviorVideoMap(UserShareReturnProfile userProfile) {
        Set<String> vidSet = new HashSet<>();
        if (null != userProfile) {
            for (List<UserSRBO> list : Arrays.asList(userProfile.getM_s_s(), userProfile.getM_r_s(), userProfile.getL_s_s(), userProfile.getL_r_s())) {
                if (null != list) {
                    for (UserSRBO u : list) {
                        if (null != u) {
                            vidSet.add(u.getId() + "");
                        }
                    }
                }
            }
        }

        Map<String, Map<String, String>> historyVideoMap = new HashMap<>();
        if (!vidSet.isEmpty()) {
            Map<String, Map<String, Map<String, String>>> videoMap = featureService.getVideoBaseInfo("", new ArrayList<>(vidSet));
            if (null != videoMap && !videoMap.isEmpty()) {
                for (Map.Entry<String, Map<String, Map<String, String>>> entry : videoMap.entrySet()) {
                    String vid = entry.getKey();
                    Map<String, Map<String, String>> map = entry.getValue();
                    if (null != map && map.containsKey("alg_vid_feature_basic_info")) {
                        historyVideoMap.put(vid, map.get("alg_vid_feature_basic_info"));
                    }
                }
            }
        }
        return historyVideoMap;
    }

    private Map<String, String> getUserFeature(long currentMs, RankParam param, Map<String, String> headInfo, UserShareReturnProfile userProfile, Map<String, Map<String, String>> userOriginInfo) {
        Map<String, Double> featMap = new HashMap<>();
        // context feature
        String appType = String.valueOf(param.getAppType());
        String hotSceneType = String.valueOf(param.getHotSceneType());
        FeatureV6.getContextFeature(currentMs, appType, hotSceneType, featMap);

        // head video feature
        FeatureV6.getVideoBaseFeature("h", currentMs, headInfo, featMap);

        // user feature
        Map<String, String> baseInfo = getUserBaseInfo(param);
        FeatureV6.getUserFeature(userOriginInfo, featMap);
        FeatureV6.getUserProfileFeature(userProfile, baseInfo, featMap);

        return FeatureBucketUtils.noBucketFeature(featMap);
    }

    private Map<String, String> getVideoFeature(long currentMs, String vid,
                                                UserShareReturnProfile userProfile,
                                                Map<String, String> headInfo, Map<String, String> rankInfo,
                                                Map<String, Map<String, String[]>> c7Map,
                                                Map<String, Map<String, String[]>> c8Map,
                                                Map<String, Map<String, String>> userOriginInfo,
                                                Map<String, Map<String, String>> historyVideoMap,
                                                Map<String, Map<String, Map<String, String>>> videoOriginInfo) {
        Map<String, Double> featMap = new HashMap<>();
        // user & video feature
        FeatureV6.getUserTagsCrossVideoFeature("c5", rankInfo, userOriginInfo.get("alg_mid_feature_return_tags"), featMap);
        FeatureV6.getUserTagsCrossVideoFeature("c6", rankInfo, userOriginInfo.get("alg_mid_feature_share_tags"), featMap);
        FeatureV6.getUserCFFeature("c7", vid, c7Map, featMap);
        FeatureV6.getUserCFFeature("c8", vid, c8Map, featMap);

        // rank video feature
        FeatureV6.getVideoBaseFeature("r", currentMs, rankInfo, featMap);
        FeatureV6.getVideoFeature(vid, videoOriginInfo, featMap);

        // head&rank cross feature
        FeatureV6.getHeadRankVideoCrossFeature(headInfo, rankInfo, featMap);

        // user profile & rank cross
        FeatureV6.getProfileVideoCrossFeature(currentMs, userProfile, rankInfo, historyVideoMap, featMap);

        return FeatureBucketUtils.noBucketFeature(featMap);
    }

    private void batchGetVideoFeature(long currentMs,
                                      UserShareReturnProfile userProfile,
                                      Map<String, String> headInfo,
                                      Map<String, Map<String, Map<String, String>>> videoBaseInfoMap,
                                      Map<String, Map<String, String[]>> c7Map,
                                      Map<String, Map<String, String[]>> c8Map,
                                      Map<String, Map<String, String>> userOriginInfo,
                                      Map<String, Map<String, String>> historyVideoMap,
                                      Map<String, Map<String, Map<String, String>>> videoOriginInfo,
                                      List<RankItem> rankItems) {
        if (null != rankItems && !rankItems.isEmpty()) {
            List<Future<Integer>> futures = new ArrayList<>();
            for (RankItem item : rankItems) {
                String vid = item.getVideoId() + "";
                Map<String, String> rankInfo = videoBaseInfoMap.getOrDefault(vid, new HashMap<>()).getOrDefault("alg_vid_feature_basic_info", new HashMap<>());
                Future<Integer> future = ThreadPoolFactory.defaultPool().submit(() -> {
                    item.featureMap = getVideoFeature(currentMs, vid, userProfile, headInfo, rankInfo, c7Map, c8Map, userOriginInfo, historyVideoMap, videoOriginInfo);
                    item.norFeatureMap = item.featureMap;
                    return 1;
                });
                futures.add(future);
            }

            try {
                for (Future<Integer> future : futures) {
                    future.get(1000, TimeUnit.MILLISECONDS);
                }
            } catch (Exception e) {
                log.error("get feature error", e);
            }
        }
    }

    private Map<String, String> getUserBaseInfo(RankParam param) {
        Map<String, String> baseInfo = new HashMap<>();
        String province = param.getProvince();
        if (null != province && !province.isEmpty()) {
            baseInfo.put("province", province.replaceAll("省$", ""));
        }

        String city = param.getCity();
        if (null != city && !city.isEmpty()) {
            baseInfo.put("city", city.replaceAll("市$", ""));
        }

        MachineInfo machineInfo = param.getMachineInfo();
        if (null != machineInfo) {
            String model = machineInfo.getModel();
            if (null != model && !model.isEmpty()) {
                baseInfo.put("model", model);
            }
            String brand = machineInfo.getBrand();
            if (null != brand && !brand.isEmpty()) {
                baseInfo.put("brand", brand);
            }
            String system = machineInfo.getSystem();
            if (null != system && !system.isEmpty()) {
                baseInfo.put("system", system);
            }
        }
        return baseInfo;
    }

    private double norPowerCalibration(double weight, double exp, double score) {
        double newScore = weight * Math.pow(score, exp);
        if (newScore > 100) {
            newScore = 100;
        } else if (newScore < score) {
            newScore = score;
        }
        return newScore;
    }

    private void addRecall(RankParam param, int recallNum, String recallName, Set<Long> setVideo, List<Video> rovRecallRank) {
        if (recallNum > 0) {
            List<Video> list = extractAndSort(param, recallName);
            list = list.stream().filter(r -> !setVideo.contains(r.getVideoId())).collect(Collectors.toList());
            list = list.subList(0, Math.min(recallNum, list.size()));
            rovRecallRank.addAll(list);
            setVideo.addAll(list.stream().map(Video::getVideoId).collect(Collectors.toSet()));
        }
    }
}
