package com.tzld.piaoquan.recommend.server.service.rank.strategy;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.FeatureService;
import com.tzld.piaoquan.recommend.server.service.rank.RankParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class RankStrategy4RegionMergeModelV566 extends RankStrategy4RegionMergeModelBasic {
    @ApolloJsonValue("${rank.score.merge.weightv566:}")
    private Map<String, Double> mergeWeight;

    @Autowired
    private RankStrategy4RegionMergeModelV563 modelV563Service;

    @Autowired
    private FeatureService featureService;

    @Override
    public List<Video> mergeAndRankRovRecall(RankParam param) {
        Map<String, Double> mergeWeight = this.mergeWeight != null ? this.mergeWeight : new HashMap<>(0);

        List<Video> result = modelV563Service.mergeAndRankRovRecall(param);
        if (null != result && !result.isEmpty()) {
            try {
                Integer versionCode = param.getVersionCode();
                int orderVersionCode = mergeWeight.getOrDefault("orderVersionCode", 1500D).intValue();
                if (null != versionCode && versionCode == orderVersionCode) {
                    int minBid = mergeWeight.getOrDefault("minBid", 39D).intValue();
                    int keepPageNum = mergeWeight.getOrDefault("keepPageNum", 1D).intValue();
                    if (highlyActiveUser(param.getMid(), minBid)) {
                        int group1Size = mergeWeight.getOrDefault("group1Size", 4D).intValue();
                        int group2Size = mergeWeight.getOrDefault("group2Size", 8D).intValue();
                        int group3Size = mergeWeight.getOrDefault("group3Size", 0D).intValue();
                        return resort(group1Size, group2Size, group3Size, result);
                    }
                }
            } catch (Exception e) {
                log.error("diversity resort error", e);
            }
        }
        return result;
    }

    private List<Video> resort(int group1Size, int group2Size, int group3Size, List<Video> videos) {
        // split
        List<Video> group1 = new ArrayList<>();
        List<Video> group2 = new ArrayList<>();
        List<Video> group3 = new ArrayList<>();
        List<Video> group4 = new ArrayList<>();
        for (int i = 0; i < videos.size(); i++) {
            if (i < group1Size) {
                group1.add(videos.get(i));
            } else if (i < group2Size) {
                group2.add(videos.get(i));
            } else if (i < group3Size) {
                group3.add(videos.get(i));
            } else {
                group4.add(videos.get(i));
            }
        }

        // merge
        List<List<Video>> groupList = new ArrayList<>();
        groupList.add(group1);
        groupList.add(group2);
        groupList.add(group3);
        List<Video> result = fillVideo(groupList);
        result.addAll(group4);
        return result;
    }

    private List<Video> fillVideo(List<List<Video>> groupList) {
        List<Video> result = new ArrayList<>();
        int maxSize = getMaxSize(groupList);
        for (int i = 0; i < maxSize; i++) {
            for (List<Video> group : groupList) {
                if (null != group && i < group.size()) {
                    result.add(group.get(i));
                }
            }
        }
        return result;
    }

    private int getMaxSize(List<List<Video>> groupList) {
        int maxSize = 0;
        if (null != groupList) {
            for (List<Video> group : groupList) {
                if (null != group) {
                    maxSize = Math.max(maxSize, group.size());
                }
            }
        }
        return maxSize;
    }

    private boolean highlyActiveUser(String mid, int minBid) {
        Map<String, String> userInfo = featureService.getUserInfo("alg_recsys_feature_user_active_level", mid);
        if (null != userInfo && userInfo.containsKey("bid")) {
            int bid = Integer.parseInt(userInfo.get("bid"));
            return bid >= minBid;
        }
        return false;
    }
}
